@@ -70,7 +70,7 @@ class DwCAReader(object):
     def __enter__(self):
         return self
 
-    def __exit__(self, type, value, traceback):
+    def __exit__(self, exc_type, exc_value, exc_traceback):
         self.close()
 
     def __init__(self, path, extensions_to_ignore=None):
@@ -92,8 +92,8 @@ class DwCAReader(object):
         try:
             self.descriptor = ArchiveDescriptor(self.open_included_file(METAFILE_NAME).read(),
                                                 files_to_ignore=extensions_to_ignore)
-        except IOError as e:
-            if e.errno == ENOENT:
+        except IOError as exc:
+            if exc.errno == ENOENT:
                 self.descriptor = None
 
         #: A :class:`xml.etree.ElementTree.Element` instance containing the (scientific) metadata
@@ -115,10 +115,10 @@ class DwCAReader(object):
         else:  # Archive without descriptor, we'll have to find and inspect the data file
             try:
                 datafile_name = self._is_valid_simple_archive()
-                d = DataFileDescriptor.make_from_file(os.path.join(self._workin_directory_path, datafile_name))
+                descriptor = DataFileDescriptor.make_from_file(os.path.join(self._workin_directory_path, datafile_name))
 
                 self._corefile = CSVDataFile(work_directory=self._workin_directory_path,
-                                             file_descriptor=d)
+                                             file_descriptor=descriptor)
                 self._extensionfiles = []
             except InvalidSimpleArchive:
                 msg = "No metafile was found, but archive includes multiple files/directories."
@@ -135,9 +135,39 @@ class DwCAReader(object):
                     r[key] = self._parse_xml_included_file(os.path.join(SOURCE_METADATA_DIRECTORY, f))
         return r
 
+    def orphaned_extension_rows(self):
+        """Return a dict of the orphaned extension rows.
+
+        Orphaned extension rows are extension rows who reference non-existing core rows. This methods returns a dict
+        such as: { 'description.txt': {u'5': [3, 4], u'6': [5]},
+                   'vernacularname.txt': {u'7': [4]}}
+
+        (meaning: in description.txt, rows at position 3 and 4 reference a core row whose ID is '5', but such a core
+        row doesn't exists.
+
+        """
+        if len(self._extensionfiles) > 0:
+
+            temp_ids = {}
+            for row in self:
+                temp_ids[row.id] = 1
+            ids = temp_ids.keys()
+
+            indexes = {}
+            for extension in self._extensionfiles:
+                coreid_index = extension.coreid_index.copy()
+                for id in ids:
+                    coreid_index.pop(id, None)
+                indexes[extension.file_descriptor.file_location] = coreid_index
+
+            return indexes
+
+        else:
+            return {}
+
     @property
     def use_extensions(self):
-        """Return True if the Archive makes use of extensions."""
+        """Return `True` if the Archive makes use of extensions."""
         return (self.descriptor is not None) and self.descriptor.extensions
 
     @property
@@ -152,8 +182,8 @@ class DwCAReader(object):
         """
         return list(self)
 
-    def get_row_by_id(self, row_id):
-        """Return the (core) row whose id is row_id.
+    def get_corerow_by_id(self, row_id):
+        """Return the (core) row whose ID is row_id.
 
         :returns:  :class:`dwca.rows.CoreRow` -- the matching row.
         :raises: :class:`dwca.exceptions.RowNotFound`
@@ -164,16 +194,28 @@ class DwCAReader(object):
             1) Not all Darwin Core Archives specifies row IDs.
             2) Nothing guarantees that the ID will actually be unique within the archive (depends
             of the data publisher). In that case, this method don't guarantee which one will be
-            returned. :meth:`.get_row_by_index` may be more appropriate in this case.
+            returned. :meth:`.get_corerow_by_position` may be more appropriate in this case.
 
         """
         for row in self:
             if row.id == str(row_id):
                 return row
-        else:
-            raise RowNotFound
 
-    def get_row_by_index(self, index):
+        raise RowNotFound
+
+    def get_row_by_id(self, row_id):
+        """Return the (core) row whose id is row_id.
+
+        .. warning::
+
+            Deprecated: this method has been renamed to :meth:`get_corerow_by_id`.
+
+        """
+        import warnings
+        warnings.warn("This method has been renamed to get_corerow_by_id().", DeprecationWarning)
+        return self.get_corerow_by_id(row_id)
+
+    def get_corerow_by_position(self, index):
         """Return a core row according to its position/index in core file.
 
         :returns:  :class:`dwca.rows.CoreRow` -- the matching row.
@@ -183,14 +225,26 @@ class DwCAReader(object):
 
             - First row has index 0
             - If index is bigger than the length of the archive, None is returned
-            - The index is often an appropriate way to unambiguously identify a core row in a DwCA.
+            - The position is often an appropriate way to unambiguously identify a core row in a DwCA.
 
         """
         for (i, row) in enumerate(self):
             if i == index:
                 return row
-        else:
-            raise RowNotFound
+
+        raise RowNotFound
+
+    def get_row_by_index(self, index):
+        """Return a core row according to its position/index in core file.
+
+        .. warning::
+
+            Deprecated: this method has been renamed to :meth:`get_corerow_by_position`.
+
+        """
+        import warnings
+        warnings.warn("This method has been renamed to get_corerow_by_position().", DeprecationWarning)
+        return self.get_corerow_by_position(index)
 
     def absolute_temporary_path(self, relative_path):
         """Return the absolute path of the file located at relative_path within the archive.
@@ -211,7 +265,7 @@ class DwCAReader(object):
         # (one single data file + possibly some metadata), returns the name of the data file.
         #
         # Otherwise, throws an InvalidSimpleArchive exception.
-        path, dirs, files = next(os.walk(self._workin_directory_path))
+        _, _, files = next(os.walk(self._workin_directory_path))
 
         if len(files) == 1:  # We found a single file
             return files[0]
@@ -219,13 +273,8 @@ class DwCAReader(object):
             # Two files found: if one of them is EML.xml, the other is considered as the data file
             if DEFAULT_METADATA_FILENAME in files:
                 return [f for f in files if f is not DEFAULT_METADATA_FILENAME][0]
-            else:
-                invalid = True
-        else:
-            invalid = True
 
-        if invalid:
-            raise InvalidSimpleArchive()
+        raise InvalidSimpleArchive()
 
     # TODO: Document: Win won't be able to cleanup if some files are not closed
     def open_included_file(self, relative_path, *args, **kwargs):
@@ -244,13 +293,13 @@ class DwCAReader(object):
         """
         # If the archive has descriptor, look for the metadata filename there.
         if self.descriptor and self.descriptor.metadata_filename:
-            fn = self.descriptor.metadata_filename
+            filename = self.descriptor.metadata_filename
 
             try:
-                return self._parse_xml_included_file(fn)
-            except IOError as e:
-                if e.errno == ENOENT:  # File not found
-                    msg = "{} is referenced in the archive descriptor but missing.".format(fn)
+                return self._parse_xml_included_file(filename)
+            except IOError as exc:
+                if exc.errno == ENOENT:  # File not found
+                    msg = "{} is referenced in the archive descriptor but missing.".format(filename)
                     raise InvalidArchive(msg)
 
         else:  # Otherwise, the metadata file has to be named 'EML.xml'
@@ -315,7 +364,8 @@ class DwCAReader(object):
         """
         #  Windows can't remove a dir with opened files
         self._corefile.close()
-        [f.close() for f in self._extensionfiles]
+        for extension_file in self._extensionfiles:
+            extension_file.close()
 
         if self._directory_to_clean:
             self._cleanup_temporary_dir()
@@ -343,8 +393,8 @@ class DwCAReader(object):
 
             self._corefile_pointer = self._corefile_pointer + 1
             return r
-        else:
-            raise StopIteration
+
+        raise StopIteration
 
 
 class GBIFResultsReader(DwCAReader):
