@@ -4,6 +4,7 @@ import sys
 import unittest
 import os
 import tempfile
+import warnings
 
 import xml.etree.ElementTree as ET
 
@@ -16,11 +17,12 @@ from dwca.descriptors import ArchiveDescriptor
 from .helpers import (GBIF_RESULTS_PATH, BASIC_ARCHIVE_PATH, EXTENSION_ARCHIVE_PATH,
                       MULTIEXTENSIONS_ARCHIVE_PATH, NOHEADERS1_PATH, NOHEADERS2_PATH,
                       IDS_ARCHIVE_PATH, DEFAULT_VAL_PATH, UTF8EOL_ARCHIVE_PATH,
-                      DIRECTORY_ARCHIVE_PATH, DEFAULT_META_VALUES, INVALID_LACKS_METADATA,
-                      SUBDIR_ARCHIVE_PATH, SIMPLE_CSV, SIMPLE_CSV_EML, SIMPLE_CSV_DOS,
-                      BASIC_ENCLOSED_ARCHIVE_PATH, INVALID_SIMPLE_TOOMUCH, INVALID_SIMPLE_TWO,
-                      SIMPLE_CSV_NOTENCLOSED, NOMETADATA_PATH, DEFAULT_METADATA_FILENAME,
-                      BASIC_ARCHIVE_TGZ_PATH, INVALID_DESCRIPTOR)
+                      DIRECTORY_ARCHIVE_PATH, DIRECTORY_CSV_QUOTE_ARCHIVE_PATH, DEFAULT_META_VALUES,
+                      INVALID_LACKS_METADATA, SUBDIR_ARCHIVE_PATH, SIMPLE_CSV, SIMPLE_CSV_EML,
+                      SIMPLE_CSV_DOS, BASIC_ENCLOSED_ARCHIVE_PATH, INVALID_SIMPLE_TOOMUCH,
+                      INVALID_SIMPLE_TWO, SIMPLE_CSV_NOTENCLOSED, NOMETADATA_PATH,
+                      DEFAULT_METADATA_FILENAME, BASIC_ARCHIVE_TGZ_PATH, INVALID_DESCRIPTOR,
+                      DWCA_ORPHANED_ROWS)
 
 
 class TestDwCAReader(unittest.TestCase):
@@ -34,7 +36,7 @@ class TestDwCAReader(unittest.TestCase):
             f = dwca.open_included_file('occurrence.txt')
 
             raw_occ = f.read()
-            self.assertTrue(raw_occ.endswith('betta splendens\n'))
+            self.assertTrue(raw_occ.endswith("'betta' splendens\n"))
 
         # TODO: test more cases: opening mode, exceptions raised, ...
 
@@ -161,7 +163,7 @@ class TestDwCAReader(unittest.TestCase):
             # Ensure we get the correct number of rows
             self.assertEqual(len(dwca.rows), 3)
             # Ensure we can access arbitrary data
-            self.assertEqual(dwca.get_row_by_index(1).data['decimallatitude'], '-31.98333')
+            self.assertEqual(dwca.get_corerow_by_position(1).data['decimallatitude'], '-31.98333')
             # Archive descriptor should be None
             self.assertIsNone(dwca.descriptor)
             # (scientific) metadata should be None
@@ -172,7 +174,7 @@ class TestDwCAReader(unittest.TestCase):
             # Ensure we get the correct number of rows
             self.assertEqual(len(dwca.rows), 3)
             # Ensure we can access arbitrary data
-            self.assertEqual(dwca.get_row_by_index(1).data['decimallatitude'], '-31.98333')
+            self.assertEqual(dwca.get_corerow_by_position(1).data['decimallatitude'], '-31.98333')
             # Archive descriptor should be None
             self.assertIsNone(dwca.descriptor)
             # (scientific) metadata should be None
@@ -183,7 +185,7 @@ class TestDwCAReader(unittest.TestCase):
             # Ensure we get the correct number of rows
             self.assertEqual(len(dwca.rows), 3)
             # Ensure we can access arbitrary data
-            self.assertEqual(dwca.get_row_by_index(1).data['decimallatitude'], '-31.98333')
+            self.assertEqual(dwca.get_corerow_by_position(1).data['decimallatitude'], '-31.98333')
             # Archive descriptor should be None
             self.assertIsNone(dwca.descriptor)
             # (scientific) metadata should be None
@@ -202,7 +204,7 @@ class TestDwCAReader(unittest.TestCase):
             # Ensure we get the correct number of rows
             self.assertEqual(len(dwca.rows), 3)
             # Ensure we can access arbitrary data
-            self.assertEqual(dwca.get_row_by_index(1).data['decimallatitude'], '-31.98333')
+            self.assertEqual(dwca.get_corerow_by_position(1).data['decimallatitude'], '-31.98333')
             # Archive descriptor should be None
             self.assertIsNone(dwca.descriptor)
             # (scientific) metadata is found
@@ -221,6 +223,21 @@ class TestDwCAReader(unittest.TestCase):
             for row in dwca:
                 self.assertIsInstance(row, CoreRow)
 
+    def test_csv_quote_dir_archive(self):
+        """If the field separator is in a quoted field, don't break on it."""
+        with DwCAReader(DIRECTORY_CSV_QUOTE_ARCHIVE_PATH) as dwca:
+            rows = list(dwca)
+            self.assertEqual(len(rows), 2)
+            self.assertEqual(rows[0].data[qn('basisOfRecord')], 'Observation, something')
+
+    def test_dont_enclose_unenclosed(self):
+        """If fields_enclosed_by is an empty string, don't enclose (even if quotes are present)"""
+        with DwCAReader(DIRECTORY_ARCHIVE_PATH) as dwca:
+            rows = list(dwca)
+
+            self.assertEqual('"betta" splendens', rows[2].data[qn('scientificName')])
+            self.assertEqual("'betta' splendens", rows[3].data[qn('scientificName')])
+
     def test_tgz_archives(self):
         """Ensure the reader (basic features) works with a .tgz Archive."""
         with DwCAReader(BASIC_ARCHIVE_TGZ_PATH) as dwca:
@@ -455,56 +472,100 @@ class TestDwCAReader(unittest.TestCase):
             # The second time, we can still find 4 rows...
             self.assertEqual(4, len([l for l in dwca]))
 
-    def test_get_row_by_index(self):
-        """Test the get_row_by_index() method work as expected"""
+    def test_deprecated_get_row_by_id(self):
+        """get_row_by_id() has been renamed get_corerow_by_id(). Make sure it still works, w/ warning."""
+
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always", DeprecationWarning)
+
+            with DwCAReader(IDS_ARCHIVE_PATH) as dwca:
+                # Passed as an integer, conversion will be tried...
+                r = dwca.get_row_by_id(3)
+                self.assertEqual('Peliperdix', r.data['http://rs.tdwg.org/dwc/terms/genus'])
+
+                self.assertEqual(1, len(w))  # Warning was issued
+                the_warning = w[0]
+                assert issubclass(the_warning.category, DeprecationWarning)
+                self.assertEqual("This method has been renamed to get_corerow_by_id().", str(the_warning.message))
+
+
+    def test_deprecated_row_by_position(self):
+        """get_row_by_index() has been renamed get_corerow_by_position(). Make sure it still works, w/ warning."""
+
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always", DeprecationWarning)
+
+            # Copy-pasted code from the long term test_get_corerow_by_position()
+            with DwCAReader(IDS_ARCHIVE_PATH) as dwca:
+                # Row IDs are ordered like this in core: id 4-1-3-2
+                first_row = dwca.get_row_by_index(0)
+                self.assertEqual(4, int(first_row.id))
+
+                self.assertEqual(1, len(w))  # Warning was issued
+                the_warning = w[0]
+                assert issubclass(the_warning.category, DeprecationWarning)
+                self.assertEqual("This method has been renamed to get_corerow_by_position().", str(the_warning.message))
+
+                last_row = dwca.get_row_by_index(3)
+                self.assertEqual(2, int(last_row.id))
+
+                # Exception raised if bigger than archive (last index: 3)
+                with self.assertRaises(RowNotFound):
+                    dwca.get_row_by_index(4)
+
+                with self.assertRaises(RowNotFound):
+                    dwca.get_row_by_index(1000)
+
+    def test_get_corerow_by_position(self):
+        """Test the get_corerow_by_position() method work as expected"""
         with DwCAReader(IDS_ARCHIVE_PATH) as dwca:
             # Row IDs are ordered like this in core: id 4-1-3-2
-            first_row = dwca.get_row_by_index(0)
+            first_row = dwca.get_corerow_by_position(0)
             self.assertEqual(4, int(first_row.id))
 
-            last_row = dwca.get_row_by_index(3)
+            last_row = dwca.get_corerow_by_position(3)
             self.assertEqual(2, int(last_row.id))
 
             # Exception raised if bigger than archive (last index: 3)
             with self.assertRaises(RowNotFound):
-                dwca.get_row_by_index(4)
+                dwca.get_corerow_by_position(4)
 
             with self.assertRaises(RowNotFound):
-                dwca.get_row_by_index(1000)
+                dwca.get_corerow_by_position(1000)
 
-    def test_get_row_by_id_string(self):
+    def test_get_corerow_by_id_string(self):
         genus_qn = 'http://rs.tdwg.org/dwc/terms/genus'
 
         with DwCAReader(IDS_ARCHIVE_PATH) as dwca:
             # Number can be passed as a string....
-            r = dwca.get_row_by_id('3')
+            r = dwca.get_corerow_by_id('3')
             self.assertEqual('Peliperdix', r.data[genus_qn])
 
-    def test_get_row_by_id_multiple_calls(self):
+    def test_get_corerow_by_id_multiple_calls(self):
         genus_qn = 'http://rs.tdwg.org/dwc/terms/genus'
 
         with DwCAReader(IDS_ARCHIVE_PATH) as dwca:
-            r = dwca.get_row_by_id('3')
+            r = dwca.get_corerow_by_id('3')
             self.assertEqual('Peliperdix', r.data[genus_qn])
 
             # If iterator is not properly reset, None will be returned
             # the second time
-            r = dwca.get_row_by_id('3')
+            r = dwca.get_corerow_by_id('3')
             self.assertEqual('Peliperdix', r.data[genus_qn])
 
-    def test_get_row_by_id_other(self):
+    def test_get_corerow_by_id_other(self):
         genus_qn = 'http://rs.tdwg.org/dwc/terms/genus'
 
         with DwCAReader(IDS_ARCHIVE_PATH) as dwca:
             # Passed as an integer, conversion will be tried...
-            r = dwca.get_row_by_id(3)
+            r = dwca.get_corerow_by_id(3)
             self.assertEqual('Peliperdix', r.data[genus_qn])
 
     def test_get_inexistent_row(self):
-        """ Ensure get_row_by_id() raises RowNotFound if we ask it an unexistent row. """
+        """ Ensure get_corerow_by_id() raises RowNotFound if we ask it an unexistent row. """
         with DwCAReader(IDS_ARCHIVE_PATH) as dwca:
             with self.assertRaises(RowNotFound):
-                dwca.get_row_by_id(8000)
+                dwca.get_corerow_by_id(8000)
 
     def test_read_core_value(self):
         """Retrieve a simple value from core file"""
@@ -721,7 +782,7 @@ class TestDwCAReader(unittest.TestCase):
         # But it should be supported for GBIF-originating archives
         # (was previously supported with GBIFResultsReader)
         with DwCAReader(GBIF_RESULTS_PATH) as results:
-            first_row = results.get_row_by_id('607759330')
+            first_row = results.get_corerow_by_id('607759330')
             m = first_row.source_metadata
 
             self.assertIsInstance(m, ET.Element)
@@ -731,7 +792,7 @@ class TestDwCAReader(unittest.TestCase):
 
             self.assertEqual(v, 'Stanley')
 
-            last_row = results.get_row_by_id('782700656')
+            last_row = results.get_corerow_by_id('782700656')
             m = last_row.source_metadata
 
             self.assertIsInstance(m, ET.Element)
@@ -748,6 +809,28 @@ class TestDwCAReader(unittest.TestCase):
 
         invalid_origin_file.close()
 
+    def test_orphaned_extension_rows_noext(self):
+        """ orphaned_extension_rows returns {} when there's no extensions."""
+        # Archive without extensions: we expect {}
+        with DwCAReader(BASIC_ARCHIVE_PATH) as dwca:
+            self.assertEqual({}, dwca.orphaned_extension_rows())
+
+    def test_orphaned_extension_rows_no_orphans(self):
+        # Archive with extensions, but no orphaned extension rows
+
+        with DwCAReader(MULTIEXTENSIONS_ARCHIVE_PATH) as dwca:
+            expected = {'description.txt': {}, 'vernacularname.txt': {}}
+            self.assertEqual(expected, dwca.orphaned_extension_rows())
+
+    def test_orphaned_extension_rows(self):
+        # Archive with extensions and orphaned rows
+        with DwCAReader(DWCA_ORPHANED_ROWS) as dwca:
+            expected = {
+                'description.txt': {u'5': [3, 4], u'6': [5]},
+                'vernacularname.txt': {u'7': [4]}
+            }
+            self.assertEqual(expected, dwca.orphaned_extension_rows())
+
 
 if __name__ == "__main__":
     unittest.main()
